
# player.py

import upygame as upg
import playerData

sf_g1 = upg.surface.Surface(8, 8, playerData.g1Pixels)
sf_g2 = upg.surface.Surface(8, 8, playerData.g2Pixels)
sf_g3 = upg.surface.Surface(8, 8, playerData.g3Pixels)
sf_g4 = upg.surface.Surface(8, 8, playerData.g4Pixels)
sf_death = upg.surface.Surface(8, 8, playerData.g5Pixels)

sf_g1f = upg.surface.Surface(8, 8, playerData.g1fPixels)
sf_g2f = upg.surface.Surface(8, 8, playerData.g2fPixels)
sf_g3f = upg.surface.Surface(8, 8, playerData.g3fPixels)
sf_g4f = upg.surface.Surface(8, 8, playerData.g4fPixels)
sf_deathf = upg.surface.Surface(8, 8, playerData.g5fPixels)

sf_trailH = upg.surface.Surface(8, 8, playerData.trailHPixels)
sf_trailH_flip = upg.surface.Surface(8, 8, playerData.trailHFlipPixels)
sf_trailV = upg.surface.Surface(8, 8, playerData.trailVPixels)
sf_trailV_flip = upg.surface.Surface(8, 8, playerData.trailVFlipPixels)


class PlayerState:
    IDLE = 0
    RUN = 1

class Player():
    def __init__(self,xs,ys,w,h):
        self.state = PlayerState.IDLE
        self.K = 1000
        self.xs = xs
        self.ys = ys
        self.dx = 0
        self.dy = 0
        self.x = self.xs * self.K
        self.y = self.ys * self.K
        self.w = w
        self.h = h
        self.flip = False
        self.surfaces = (sf_g1,sf_g2,sf_g3,sf_g4)
        self.surfacesFlipped = (sf_g1f,sf_g2f,sf_g3f,sf_g4f)
        self.activeSurface = self.surfaces[0]
        self.frame = 0
        self.rect = upg.Rect(0,0,0,0)
        self.hitBoxRight = upg.Rect(0,0,0,0)
        self.hitBoxLeft = upg.Rect(0,0,0,0)
        self.hitBoxUp = upg.Rect(0,0,0,0)
        self.hitBoxDown = upg.Rect(0,0,0,0)
        
        
    def setXY(self,newX,newY):
        self.x = newX * self.K
        self.y = newY * self.K
    
    def update(self,inputs,level):
        #frame
        self.frame += 1
        if self.frame>1000:
            self.frame = 0
            
        if (self.dx == 0 and self.dy == 0) :
            self.state = PlayerState.IDLE
        else:
            self.state = PlayerState.RUN
            
        #
        if self.state == PlayerState.IDLE:
            if (inputs.LeftTrigger) :
                self.flip = True
                self.dx = -3500
            if (inputs.RightTrigger) :
                self.flip = False
                self.dx = 3500
            if (inputs.UpTrigger) :
                self.dy = -3500
            if (inputs.DownTrigger) :
                self.dy = 3500
        
        
            
        #calcualte hit boxes
        self.rect = self.activeSurface.get_rect()
        #up
        self.hitBoxUp.x = self.xs+2
        self.hitBoxUp.y = self.ys
        self.hitBoxUp.width = 4
        self.hitBoxUp.height = 2
        #down
        self.hitBoxDown.x = self.xs +2 
        self.hitBoxDown.y = self.ys + 6
        self.hitBoxDown.width = 4
        self.hitBoxDown.height = 2
        #left
        self.hitBoxLeft.x = self.xs
        self.hitBoxLeft.y = self.ys +  2
        self.hitBoxLeft.width = 2
        self.hitBoxLeft.height = 4
        #right
        self.hitBoxRight.x = self.xs + 6
        self.hitBoxRight.y = self.ys + 2
        self.hitBoxRight.width = 2
        self.hitBoxRight.height = 4
        
        
        #check collision separated
        #self.rect.x = self.xs 
        #self.rect.y = self.ys
        #collidedTile = level.collideWith(self.rect)
        
        #gravity
        #self.dy += 150
        
        #apply force and get screen coordinate
        oldx = self.x
        oldy = self.y
        
        newx = self.x + self.dx
        newy = self.y + self.dy


        self.rect.x = newx // self.K
        self.rect.y = newy // self.K
        
        #check DX
        hit = level.collideWith(self.hitBoxRight)
        if hit and hit.solid:
            if self.dx > 0:
                self.dx = 0
                newx = (hit.x - self.w) * self.K
            if hit.move and hit.move.dx < 0:
                newx += (hit.move.dx * self.K)
        #check SX
        hit = level.collideWith(self.hitBoxLeft)
        if hit and hit.solid:
            if self.dx < 0:
                self.dx = 0
                newx = (hit.x + hit.w) * self.K
            if hit.move and hit.move.dx > 0:
                newx += (hit.move.dx * self.K)
        #check Down
        hit = level.collideWith(self.hitBoxDown)
        if hit and hit.solid:
            if self.dy > 0:
                self.dy = 0
                newy = (hit.y - self.h) * self.K
            if hit.move and hit.move.dy < 0:
                newy += (hit.move.dy * self.K)
        #check Up
        hit = level.collideWith(self.hitBoxUp)
        if hit and hit.solid:
            if self.dy < 0:
                self.dy = 0
                newy = (hit.y + hit.h) * self.K
            if hit.move and  hit.move.dy > 0:
                newy += (hit.move.dy * self.K)
    
        #apply
        self.x = newx
        self.y = newy
        self.xs = newx // self.K
        self.ys = newy // self.K
        
        
  
        
    def draw(self,screen,cam):
        
        xc = cam.getX() + self.xs
        yc = cam.getY() + self.ys
        
        if self.dx>0:
            screen.blit(sf_trailH,xc - self.w, yc)
        if self.dx<0:
            screen.blit(sf_trailH_flip,xc + self.w, yc)
        if self.dy>0:
            screen.blit(sf_trailV,xc, yc - self.h)
        if self.dy<0:
            screen.blit(sf_trailV_flip,xc, yc + self.w)
        
        
        if self.state == PlayerState.IDLE:
            if self.flip:
                self.activeSurface = self.surfacesFlipped[0]
            else:
                self.activeSurface = self.surfaces[0]
        elif self.state == PlayerState.RUN:
            if self.flip:
                self.activeSurface = self.surfacesFlipped[self.frame//3%4]
            else:
                self.activeSurface = self.surfaces[self.frame//3%4]
        
        #draw
        screen.blit(self.activeSurface, xc , yc)
        #screen.fill(4,self.hitBoxUp)
        #screen.fill(8,self.hitBoxDown)
        #screen.fill(10,self.hitBoxLeft)
        #screen.fill(13,self.hitBoxRight)
        
         
    